@@ -1,12 +1,10 @@
 from __future__ import annotations
 
-from typing import TypeVar, overload
+from typing import overload
 
-__all__ = [
-    'Coordinate',
-    'Position',
-    'Size'
-]
+from typing_extensions import Self
+
+__all__ = ["Coordinate", "Position", "Size"]
 
 
 class Coordinate:
@@ -23,14 +21,12 @@ class Coordinate:
     """Vertical coordinate."""
 
     @overload
-    def __init__(self: SelfCoord, other: tuple[int, int] | SelfCoord, /) -> None:
-        ...
+    def __init__(self, other: tuple[int, int] | Self, /) -> None: ...
 
     @overload
-    def __init__(self: SelfCoord, x: int, y: int, /) -> None:
-        ...
+    def __init__(self, x: int, y: int, /) -> None: ...
 
-    def __init__(self: SelfCoord, x_or_self: int | tuple[int, int] | SelfCoord, y: int | None = None, /) -> None:
+    def __init__(self, x_or_self: int | tuple[int, int] | Self, y: int | None = None, /) -> None:
         from ..exceptions import CustomValueError
 
         if isinstance(x_or_self, int):
@@ -48,9 +44,6 @@ class Coordinate:
         self.y = y
 
 
-SelfCoord = TypeVar('SelfCoord', bound=Coordinate)
-
-
 class Position(Coordinate):
     """Positive set of an (x,y) offset relative to the top left corner of an area."""
 
