@@ -3,38 +3,48 @@ from __future__ import annotations
 from functools import partial, wraps
 from inspect import Signature
 from inspect import _empty as empty_param
-from inspect import isclass
 from typing import (
-    TYPE_CHECKING, Any, Callable, Concatenate, Generator, Generic, Iterable, Iterator, Mapping,
-    NoReturn, Protocol, Sequence, TypeVar, cast, overload
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    ClassVar,
+    Concatenate,
+    Generator,
+    Generic,
+    Iterable,
+    Iterator,
+    Mapping,
+    NoReturn,
+    Protocol,
+    Sequence,
+    cast,
+    overload,
 )
 
-from .builtins import F0, F1, P0, P1, R0, R1, T0, T1, T2, KwargsT, P, R, T
+from typing_extensions import Self
+
+from .builtins import F0, F1, P0, P1, R0, R1, T0, T1, T2, KwargsT, P, R, R0_co, R1_co, R_co, T, T0_co, T1_co, T_co
 
 __all__ = [
-    'copy_signature',
-
-    'inject_self',
-
-    'inject_kwargs_params',
-
-    'complex_hash',
-
-    'get_subclasses',
-
-    'classproperty', 'cachedproperty',
-
-    'KwargsNotNone',
-
-    'Singleton', 'to_singleton',
-
-    'LinearRangeLut'
+    "KwargsNotNone",
+    "LinearRangeLut",
+    "Singleton",
+    "cachedproperty",
+    "classproperty",
+    "complex_hash",
+    "copy_signature",
+    "get_subclasses",
+    "inject_kwargs_params",
+    "inject_self",
+    "to_singleton",
 ]
+# ruff: noqa: N801
 
 
 class copy_signature(Generic[F0]):
     """
-    Type util to copy the signature of one function to another function.\n
+    Type util to copy the signature of one function to another function.
+
     Especially useful for passthrough functions.
 
     .. code-block::
@@ -66,42 +76,31 @@ class copy_signature(Generic[F0]):
         return cast(F0, wrapped)
 
 
-class injected_self_func(Generic[T, P, R], Protocol):  # type: ignore[misc]
+class injected_self_func(Protocol[T_co, P, R_co]):
     @overload
     @staticmethod
-    def __call__(*args: P.args, **kwargs: P.kwargs) -> R:
+    def __call__(*args: P.args, **kwargs: P.kwargs) -> R_co: ...
+
+    @overload
+    @staticmethod
+    def __call__(self: T_co, *args: P.args, **kwargs: P.kwargs) -> R_co:  # type: ignore[misc]
         ...
 
     @overload
     @staticmethod
-    def __call__(self: T, *args: P.args, **kwargs: P.kwargs) -> R:
-        ...
-
-    @overload
-    @staticmethod
-    def __call__(self: T, _self: T, *args: P.args, **kwargs: P.kwargs) -> R:
-        ...
-
-    @overload
-    @staticmethod
-    def __call__(cls: type[T], *args: P.args, **kwargs: P.kwargs) -> R:
-        ...
-
-    @overload
-    @staticmethod
-    def __call__(cls: type[T], _cls: type[T], *args: P.args, **kwargs: P.kwargs) -> R:
-        ...
-
-    @staticmethod
-    def __call__(*args: Any, **kwds: Any) -> Any:
-        ...
+    def __call__(cls: type[T_co], *args: P.args, **kwargs: P.kwargs) -> R_co: ...
 
 
-self_objects_cache = dict[type[T], T]()  # type: ignore
+self_objects_cache = dict[Any, Any]()
 
 
-class inject_self_base(Generic[T, P, R]):
-    def __init__(self, function: Callable[Concatenate[T, P], R], /, *, cache: bool = False) -> None:
+class inject_self_base(Generic[T_co, P, R_co]):
+    cache: bool | None
+    signature: Signature | None
+    init_kwargs: list[str] | None
+    first_key: str | None
+
+    def __init__(self, function: Callable[Concatenate[T_co, P], R_co], /, *, cache: bool = False) -> None:
         """
         Wrap ``function`` to always have a self provided to it.
 
@@ -124,42 +123,38 @@ class inject_self_base(Generic[T, P, R]):
         self.clean_kwargs = False
 
     def __get__(
-        self, class_obj: type[T] | T | None, class_type: type[T] | type[type[T]]  # type: ignore
-    ) -> injected_self_func[T, P, R]:
-        if not self.signature or not self.first_key:  # type: ignore
-            self.signature = Signature.from_callable(self.function, eval_str=True)  # type: ignore
-            self.first_key = next(iter(list(self.signature.parameters.keys())), None)  # type: ignore
+        self,
+        class_obj: type[T] | T | None,
+        class_type: type[T | type[T]] | Any,  # type: ignore[valid-type]
+    ) -> injected_self_func[T_co, P, R_co]:
+        if not self.signature or not self.first_key:
+            self.signature = Signature.from_callable(self.function, eval_str=True)
+            self.first_key = next(iter(list(self.signature.parameters.keys())), None)
 
             if isinstance(self, inject_self.init_kwargs):
                 from ..exceptions import CustomValueError
 
-                if 4 not in {x.kind for x in self.signature.parameters.values()}:  # type: ignore
+                if 4 not in {x.kind for x in self.signature.parameters.values()}:
                     raise CustomValueError(
-                        'This function hasn\'t got any kwargs!', 'inject_self.init_kwargs', self.function
+                        "This function hasn't got any kwargs!", "inject_self.init_kwargs", self.function
                     )
 
-                self.init_kwargs = list[str](  # type: ignore
-                    k for k, x in self.signature.parameters.items() if x.kind != 4  # type: ignore
-                )
+                self.init_kwargs = list[str](k for k, x in self.signature.parameters.items() if x.kind != 4)
 
         @wraps(self.function)
         def _wrapper(*args: Any, **kwargs: Any) -> Any:
-            first_arg = (args[0] if args else None) or (
-                kwargs.get(self.first_key, None) if self.first_key else None
-            )
+            first_arg = (args[0] if args else None) or (kwargs.get(self.first_key, None) if self.first_key else None)
 
-            if (
-                first_arg and (
-                    (is_obj := isinstance(first_arg, class_type))
-                    or isinstance(first_arg, type(class_type))  # noqa
-                    or first_arg is class_type  # noqa
-                )
+            if first_arg and (
+                (is_obj := isinstance(first_arg, class_type))
+                or isinstance(first_arg, type(class_type))
+                or first_arg is class_type
             ):
                 obj = first_arg if is_obj else first_arg()
                 if args:
                     args = args[1:]
                 elif kwargs and self.first_key:
-                    kwargs.pop(self.first_key)  # type: ignore
+                    kwargs.pop(self.first_key)
             elif class_obj is None:
                 if self.cache:
                     if class_type not in self_objects_cache:
@@ -167,7 +162,7 @@ class inject_self_base(Generic[T, P, R]):
                     else:
                         obj = self_objects_cache[class_type]
                 elif self.init_kwargs:
-                    obj = class_type(  # type: ignore
+                    obj = class_type(
                         *self.args, **(self.kwargs | {k: v for k, v in kwargs.items() if k not in self.init_kwargs})
                     )
                     if self.clean_kwargs:
@@ -177,12 +172,12 @@ class inject_self_base(Generic[T, P, R]):
             else:
                 obj = class_obj
 
-            return self.function(obj, *args, **kwargs)
+            return self.function(obj, *args, **kwargs)  # type: ignore
 
         return _wrapper
 
-    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> R:
-        return self.__get__(None, self)(*args, **kwargs)  # type: ignore
+    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> R_co:
+        return self.__get__(None, self)(*args, **kwargs)
 
     @property
     def __signature__(self) -> Signature:
@@ -191,7 +186,7 @@ class inject_self_base(Generic[T, P, R]):
     @classmethod
     def with_args(
         cls, *args: Any, **kwargs: Any
-    ) -> Callable[[Callable[Concatenate[T0, P0], R0]], inject_self[T0, P0, R0]]:
+    ) -> Callable[[Callable[Concatenate[T0_co, P0], R0_co]], inject_self[T0_co, P0, R0_co]]:
         """Provide custom args to instantiate the ``self`` object with."""
 
         def _wrapper(function: Callable[Concatenate[T0, P0], R0]) -> inject_self[T0, P0, R0]:
@@ -199,86 +194,83 @@ class inject_self_base(Generic[T, P, R]):
             inj.args = args
             inj.kwargs = kwargs
             return inj  # type: ignore
+
         return _wrapper
 
 
-class inject_self(Generic[T, P, R], inject_self_base[T, P, R]):
+class inject_self(inject_self_base[T_co, P, R_co]):
     """Wrap a method so it always has a constructed ``self`` provided to it."""
 
-    class cached(Generic[T0, P0, R0], inject_self_base[T0, P0, R0]):
+    class cached(inject_self_base[T0_co, P0, R0_co]):
         """
         Wrap a method so it always has a constructed ``self`` provided to it.
         Once ``self`` is constructed, it will be reused.
         """
 
-        class property(Generic[T1, R1]):
-            def __init__(self, function: Callable[[T1], R1]) -> None:
+        class property(Generic[T1_co, R1_co]):
+            def __init__(self, function: Callable[[T1_co], R1_co]) -> None:
                 self.function = inject_self(function)
 
-            def __get__(
-                self, class_obj: type[T1] | T1 | None, class_type: type[T1] | type[type[T1]]  # type: ignore
-            ) -> R1:
+            def __get__(self, class_obj: type[T1_co] | T1_co | None, class_type: type[T1_co] | T1_co) -> R1_co:
                 return self.function.__get__(class_obj, class_type)()
 
-    class init_kwargs(Generic[T0, P0, R0], inject_self_base[T0, P0, R0]):
+    class init_kwargs(inject_self_base[T0_co, P0, R0_co]):
         """
         Wrap a method so it always has a constructed ``self`` provided to it.
         When constructed, kwargs to the function will be passed to the constructor.
         """
 
         @classmethod
-        def clean(cls, function: Callable[Concatenate[T1, P1], R1]) -> inject_self[T1, P1, R1]:
+        def clean(cls, function: Callable[Concatenate[T1_co, P1], R1_co]) -> inject_self[T1_co, P1, R1_co]:
             """Wrap a method, pass kwargs to the constructor and remove them from actual **kwargs."""
             inj = cls(function)  # type: ignore
             inj.clean_kwargs = True
             return inj  # type: ignore
 
-    class property(Generic[T0, R0]):
-        def __init__(self, function: Callable[[T0], R0]) -> None:
+    class property(Generic[T0_co, R0_co]):
+        def __init__(self, function: Callable[[T0_co], R0_co]) -> None:
             self.function = inject_self(function)
 
-        def __get__(
-            self, class_obj: type[T0] | T0 | None, class_type: type[T0] | type[type[T0]]  # type: ignore
-        ) -> R0:
+        def __get__(self, class_obj: type[T0_co] | T0_co | None, class_type: type[T0_co] | T0_co) -> R0_co:
             return self.function.__get__(class_obj, class_type)()
 
 
-class inject_kwargs_params_base_func(Generic[T, P, R], Callable[Concatenate[T, P], R]):  # type: ignore[misc]
-    def __call__(self: T, *args: P.args, **kwargs: P.kwargs) -> R:  # type: ignore
-        ...
+class inject_kwargs_params_base_func(Generic[T_co, P, R_co]):
+    def __call__(self: T_co, *args: P.args, **kwargs: P.kwargs) -> R_co:
+        raise NotImplementedError
 
 
-class inject_kwargs_params_base(Generic[T, P, R]):
-    _kwargs_name = 'kwargs'
+class inject_kwargs_params_base(Generic[T_co, P, R_co]):
+    signature: Signature | None
 
-    def __init__(self, function: Callable[Concatenate[T, P], R]) -> None:
+    _kwargs_name = "kwargs"
+
+    def __init__(self, function: Callable[Concatenate[T_co, P], R_co]) -> None:
         self.function = function
 
         self.signature = None
 
-    def __get__(
-        self, class_obj: T, class_type: type[T]
-    ) -> inject_kwargs_params_base_func[T, P, R]:
+    def __get__(self, class_obj: T, class_type: type[T]) -> inject_kwargs_params_base_func[T_co, P, R_co]:
         if not self.signature:
-            self.signature = Signature.from_callable(self.function, eval_str=True)  # type: ignore
+            self.signature = Signature.from_callable(self.function, eval_str=True)
 
             if (
-                isinstance(self, inject_kwargs_params.add_to_kwargs)  # type: ignore
-                and (4 not in {x.kind for x in self.signature.parameters.values()})  # type: ignore
+                isinstance(self, inject_kwargs_params.add_to_kwargs)  # type: ignore[arg-type]
+                and (4 not in {x.kind for x in self.signature.parameters.values()})
             ):
                 from ..exceptions import CustomValueError
 
                 raise CustomValueError(
-                    'This function hasn\'t got any kwargs!', 'inject_kwargs_params.add_to_kwargs', self.function
+                    "This function hasn't got any kwargs!", "inject_kwargs_params.add_to_kwargs", self.function
                 )
 
         this = self
 
         @wraps(self.function)
-        def _wrapper(self: T, *_args: P.args, **kwargs: P.kwargs) -> R:
+        def _wrapper(self: Any, *_args: Any, **kwargs: Any) -> R_co:
             assert this.signature
 
-            if class_obj and not isinstance(self, class_type):  # type: ignore
+            if class_obj and not isinstance(self, class_type):
                 _args = (self, *_args)
                 self = class_obj
 
@@ -312,14 +304,14 @@ class inject_kwargs_params_base(Generic[T, P, R]):
 
                     kwargs[key] = kw_value
 
-            if isinstance(this, inject_kwargs_params.add_to_kwargs):
+            if isinstance(this, inject_kwargs_params.add_to_kwargs):  # type: ignore[arg-type]
                 kwargs |= this_kwargs
 
             return this.function(self, *args, **kwargs)
 
         return _wrapper  # type: ignore
 
-    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> R:
+    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> R_co:
         return self.__get__(None, self)(*args, **kwargs)  # type: ignore
 
     @property
@@ -335,21 +327,20 @@ class inject_kwargs_params_base(Generic[T, P, R]):
 
 
 if TYPE_CHECKING:  # love you mypy...
+
     class _add_to_kwargs:
-        def __call__(self, func: F1) -> F1:
-            ...
+        def __call__(self, func: F1) -> F1: ...
 
     class _inject_kwargs_params:
-        def __call__(self, func: F0) -> F0:
-            ...
+        def __call__(self, func: F0) -> F0: ...
 
         add_to_kwargs = _add_to_kwargs()
 
     inject_kwargs_params = _inject_kwargs_params()
 else:
+
     class inject_kwargs_params(Generic[T, P, R], inject_kwargs_params_base[T, P, R]):
-        class add_to_kwargs(Generic[T0, P0, R0], inject_kwargs_params_base[T0, P0, R0]):
-            ...
+        class add_to_kwargs(Generic[T0, P0, R0], inject_kwargs_params_base[T0, P0, R0]): ...
 
 
 class complex_hash(Generic[T]):
@@ -362,11 +353,7 @@ class complex_hash(Generic[T]):
     def __new__(cls, class_type: T) -> T:  # type: ignore
         class inner_class_type(class_type):  # type: ignore
             def __hash__(self) -> int:
-                return complex_hash.hash(
-                    self.__class__.__name__, *(
-                        getattr(self, key) for key in self.__annotations__.keys()
-                    )
-                )
+                return complex_hash.hash(self.__class__.__name__, *(getattr(self, key) for key in self.__annotations__))
 
         return inner_class_type  # type: ignore
 
@@ -385,14 +372,11 @@ class complex_hash(Generic[T]):
             try:
                 new_hash = hash(value)
             except TypeError:
-                if isinstance(value, Iterable):
-                    new_hash = complex_hash.hash(*value)
-                else:
-                    new_hash = hash(str(value))
+                new_hash = complex_hash.hash(*value) if isinstance(value, Iterable) else hash(str(value))
 
             values.append(str(new_hash))
 
-        return hash('_'.join(values))
+        return hash("_".join(values))
 
 
 def get_subclasses(family: type[T], exclude: Sequence[type[T]] = []) -> list[type[T]]:
@@ -416,94 +400,60 @@ def get_subclasses(family: type[T], exclude: Sequence[type[T]] = []) -> list[typ
     return list(set(_subclasses(family)))
 
 
-class classproperty(Generic[P, R, T, T0, P0]):
+class classproperty(Generic[T, R]):
     """
     Make a class property. A combination between classmethod and property.
     """
 
     __isabstractmethod__: bool = False
 
-    class metaclass(type):
-        """This must be set for the decorator to work."""
-
-        def __setattr__(self, key: str, value: Any) -> None:
-            if key in self.__dict__:
-                obj = self.__dict__.get(key)
-
-                if obj and isinstance(obj, classproperty):
-                    obj.__set__(self, value)
-                    return
-
-            super(classproperty.metaclass, self).__setattr__(key, value)
-
     def __init__(
         self,
-        fget: classmethod[T, P, R] | Callable[P, R],
-        fset: classmethod[T, P, None] | Callable[[T, T0], None] | None = None,
-        fdel: classmethod[T, P1, None] | Callable[P1, None] | None = None,
+        fget: Callable[[type[T]], R] | classmethod[T, ..., R],
+        fset: Callable[[type[T], R], None] | classmethod[T, Concatenate[R, ...], None] | None = None,
+        fdel: Callable[[type[T]], None] | classmethod[T, ..., None] | None = None,
         doc: str | None = None,
     ) -> None:
         self.fget = self._wrap(fget)
         self.fset = self._wrap(fset) if fset is not None else fset
         self.fdel = self._wrap(fdel) if fdel is not None else fdel
 
-        self.doc = doc
+        self.__doc__ = doc
 
-    def _wrap(self, func: classmethod[T1, P1, R1] | Callable[P1, R1]) -> classmethod[T1, P1, R1]:
-        if not isinstance(func, (classmethod, staticmethod)):
-            func = classmethod(func)  # type: ignore
+    def _wrap(self, func: Callable[..., R1] | classmethod[T, P1, R1]) -> classmethod[T, P1, R1]:
+        if not isinstance(func, classmethod):
+            func = classmethod(func)
 
-        return func  # type: ignore
+        return func
 
-    def getter(self, __fget: classmethod[T, P, R] | Callable[P1, R1]) -> classproperty[P1, R1, T, T0, P0]:
-        self.fget = self._wrap(__fget)  # type: ignore
-        return self  # type: ignore
+    def __get__(self, obj: T | None, type_: type | None = None) -> R:
+        if type_ is None:
+            type_ = type(obj)
 
-    def setter(self, __fset: classmethod[T1, P, None] | Callable[[T1, T2], None]) -> classproperty[P, R, T1, T2, P0]:
-        self.fset = self._wrap(__fset)  # type: ignore
-        return self  # type: ignore
-
-    def deleter(self, __fdel: classmethod[T1, P1, None] | Callable[P1, None]) -> classproperty[P, R, T, T0, P1]:
-        self.fdel = self._wrap(__fdel)  # type: ignore
-        return self  # type: ignore
-
-    def __get__(self, __obj: Any, __type: type | None = None) -> R:
-        if __type is None:
-            __type = type(__obj)
-
-        return self.fget.__get__(__obj, __type)()  # type: ignore
-
-    def __set__(self, __obj: Any, __value: T1) -> None:
-        from ..exceptions import CustomError
+        return self.fget.__get__(obj, type_)()
 
+    def __set__(self, obj: T, value: Any) -> None:
         if not self.fset:
-            raise CustomError[AttributeError]("Can't set attribute")
+            raise AttributeError(
+                f'classproperty with getter "{self.__name__}" of "{obj.__class__.__name__}" object has no setter.'
+            )
 
-        if isclass(__obj):
-            type_, __obj = __obj, None
-        else:
-            type_ = type(__obj)
-
-        return self.fset.__get__(__obj, type_)(__value)
-
-    def __delete__(self, __obj: Any) -> None:
-        from ..exceptions import CustomError
+        self.fset.__get__(None, type(obj))(value)
 
+    def __delete__(self, obj: T) -> None:
         if not self.fdel:
-            raise CustomError[AttributeError]("Can't delete attribute")
+            raise AttributeError(
+                f'classproperty with getter "{self.__name__}" of "{obj.__class__.__name__}" object has no deleter.'
+            )
 
-        if isclass(__obj):
-            type_, __obj = __obj, None
-        else:
-            type_ = type(__obj)
-
-        return self.fdel.__delete__(__obj, type_)(__obj)  # type: ignore
+        self.fdel.__get__(None, type(obj))()
 
+    @property
     def __name__(self) -> str:
         return self.fget.__name__
 
 
-class cachedproperty(property, Generic[P, R, T, T0, P0]):
+class cachedproperty(property, Generic[P, R_co, T, T0, P0]):
     """
     Wrapper for a one-time get property, that will be cached.
 
@@ -516,12 +466,13 @@ class cachedproperty(property, Generic[P, R, T, T0, P0]):
 
     __isabstractmethod__: bool = False
 
-    cache_key = '_jetpt_cachedproperty_cache'
+    cache_key = "_jetpt_cachedproperty_cache"
 
     class baseclass:
         """Inherit from this class to automatically set the cache dict."""
 
         if not TYPE_CHECKING:
+
             def __new__(cls, *args: Any, **kwargs: Any) -> None:
                 try:
                     self = super().__new__(cls, *args, **kwargs)
@@ -531,88 +482,91 @@ class cachedproperty(property, Generic[P, R, T, T0, P0]):
                 return self
 
     if TYPE_CHECKING:
+
         def __init__(
-            self, fget: Callable[P, R], fset: Callable[[T, T0], None] | None = None,
-            fdel: Callable[P0, None] | None = None, doc: str | None = None,
-        ) -> None:
-            ...
+            self,
+            fget: Callable[P, R_co],
+            fset: Callable[[T, T0], None] | None = None,
+            fdel: Callable[P0, None] | None = None,
+            doc: str | None = None,
+        ) -> None: ...
 
-        def getter(self, __fget: Callable[P1, R1]) -> cachedproperty[P1, R1, T, T0, P0]:
-            ...
+        def getter(self, fget: Callable[P1, R0_co]) -> cachedproperty[P1, R0_co, T, T0, P0]: ...
 
-        def setter(self, __fset: Callable[[T1, T2], None]) -> cachedproperty[P, R, T1, T2, P0]:
-            ...
+        def setter(self, fset: Callable[[T1, T2], None]) -> cachedproperty[P, R_co, T1, T2, P0]: ...
 
-        def deleter(self, __fdel: Callable[P1, None]) -> cachedproperty[P, R, T, T0, P1]:
-            ...
+        def deleter(self, fdel: Callable[P1, None]) -> cachedproperty[P, R_co, T, T0, P1]: ...
 
-    def __get__(self, __obj: Any, __type: type | None = None) -> R:
+    @overload
+    def __get__(self, obj: None, type_: type | None = None) -> Self: ...
+
+    @overload
+    def __get__(self, obj: object, type_: type | None = None) -> R_co: ...
+
+    def __get__(self, obj: Any, type_: type | None = None) -> Any:
         if isinstance(self.fget, classproperty):
-            function = partial(self.fget.__get__, __obj, __type)  # type: ignore
-            __obj = __type
+            function = partial(self.fget.__get__, obj, type_)  # type: ignore
+            obj = type_
 
-            if not hasattr(__obj, cachedproperty.cache_key):
-                setattr(__obj, cachedproperty.cache_key, dict[str, Any]())
+            if not hasattr(obj, cachedproperty.cache_key):
+                setattr(obj, cachedproperty.cache_key, dict[str, Any]())
 
-            cache = getattr(__obj, cachedproperty.cache_key)
+            cache = getattr(obj, cachedproperty.cache_key)
             name = self.fget.__name__
         else:
-            function = self.fget.__get__(__obj, __type)  # type: ignore
-            cache = __obj.__dict__.get(cachedproperty.cache_key)
+            assert self.fget
+            function = self.fget.__get__(obj, type_)
+            cache = obj.__dict__.get(cachedproperty.cache_key)
             name = function.__name__
 
         if name not in cache:
             cache[name] = function()
 
-        return cache[name]  # type: ignore
+        return cache[name]
+
+    if TYPE_CHECKING:
+
+        def __set__(self, obj: Any, value: R_co, /) -> None:  # type: ignore[misc]
+            ...
 
 
 class KwargsNotNone(KwargsT):
     """Remove all None objects from this kwargs dict."""
 
     if not TYPE_CHECKING:
-        def __new__(cls, *args: Any, **kwargs: Any) -> KwargsNotNone:
-            return KwargsT(**{
-                key: value for key, value in KwargsT(*args, **kwargs).items()
-                if value is not None
-            })
 
-
-SingleMeta = TypeVar('SingleMeta', bound=type)
+        def __new__(cls, *args: Any, **kwargs: Any) -> Self:
+            return KwargsT(**{key: value for key, value in KwargsT(*args, **kwargs).items() if value is not None})
 
 
 class SingletonMeta(type):
-    _instances = dict[type[SingleMeta], SingleMeta]()  # type: ignore
+    _instances: ClassVar[dict[type[Any], Any]] = {}
     _singleton_init: bool
 
-    def __new__(
-        cls: type[SingletonSelf], name: str, bases: tuple[type, ...], namespace: dict[str, Any], **kwargs: Any
-    ) -> SingletonSelf:
-        return type.__new__(cls, name, bases, namespace | {'_singleton_init': kwargs.pop('init', False)})
+    def __new__(cls, name: str, bases: tuple[type, ...], namespace: dict[str, Any], **kwargs: Any) -> SingletonMeta:
+        return type.__new__(cls, name, bases, namespace | {"_singleton_init": kwargs.pop("init", False)})
 
-    def __call__(cls: type[SingletonSelf], *args: Any, **kwargs: Any) -> SingletonSelf:  # type: ignore
+    def __call__(cls, *args: Any, **kwargs: Any) -> SingletonMeta:
         if cls not in cls._instances:
             cls._instances[cls] = super(SingletonMeta, cls).__call__(*args, **kwargs)
         elif cls._singleton_init:
-            cls._instances[cls].__init__(*args, **kwargs)  # type: ignore
+            cls._instances[cls].__init__(*args, **kwargs)
 
         return cls._instances[cls]
 
 
-SingletonSelf = TypeVar('SingletonSelf', bound=SingletonMeta)
-
-
 class Singleton(metaclass=SingletonMeta):
     """Handy class to inherit to have the SingletonMeta metaclass."""
 
 
 class to_singleton_impl:
     _ts_args = tuple[str, ...]()
-    _ts_kwargs = dict[str, Any]()
+    _ts_kwargs: Mapping[str, Any] = {}
     _add_classes = tuple[type, ...]()
 
     def __new__(_cls, cls: type[T]) -> T:  # type: ignore
         if _cls._add_classes:
+
             class rcls(cls, *_cls._add_classes):  # type: ignore
                 ...
         else:
@@ -631,11 +585,11 @@ class to_singleton_impl:
 
 class to_singleton(to_singleton_impl):
     class as_property(to_singleton_impl):
-        _add_classes = (property, )
+        _add_classes = (property,)
 
 
 class LinearRangeLut(Mapping[int, int]):
-    __slots__ = ('ranges', '_ranges_idx_lut', '_misses_n')
+    __slots__ = ("_misses_n", "_ranges_idx_lut", "ranges")
 
     def __init__(self, ranges: Mapping[int, range]) -> None:
         self.ranges = ranges
@@ -644,6 +598,8 @@ class LinearRangeLut(Mapping[int, int]):
         self._misses_n = 0
 
     def __getitem__(self, n: int) -> int:
+        missed_hit = 0
+
         for missed_hit, (idx, k) in enumerate(self._ranges_idx_lut):
             if n in k:
                 break
